module Skel_tree

	def get_instances
		ents = Sketchup.active_model.entities
		instances = ents.select{|e| e.typename == "ComponentInstance"&& e.layer.name != "master"}
		return instances
		end
	
	def create_tree
		Sketchup.active_model.definitions.purge_unused
		instances = get_instances
		# Find the instance named "body" - set it as the root node
		root_instance = instances.find {|i| i.name == "body"}
		if root_instance
			root_node = Tree::TreeNode.new(root_instance.name, root_instance)
		else
			puts "Couldn't find the body."
		end
		
		# Find children of "body" - names include "upper" and "lower"
		instances.each do |upper|
			if upper.hidden? == true
				next
			end
			# Find instances whose names start include "upper"
			if upper.name.match(/upper/)
				     
				# Add each upper ComponentInstance as a child of the root node
				upper_node = root_node.add Tree::TreeNode.new(upper.name, upper)
         
				# Find the lower ComponentInstance corresponding to the upper instance
				lower_name = upper.name.sub "upper", "lower"
				lower = instances.find{|inst| inst.name == lower_name}
         
				# Create a child node from the lower ComponentInstance
				if lower
					lower_node = upper_node.add Tree::TreeNode.new(lower.name, lower)
				end
			end
		end
		return root_node
	end

	def find_joint(instance)
		center = [0, 0, 0]
		Sketchup.active_model.definitions.each do |d|
			joint_instances = d.instances.select{|i| i.definition.name == "joint" || i.name == "joint"}
			if joint_instances
				right_joint = joint_instances.find{|j| j.parent.name == instance.definition.name}
				if right_joint
					center = right_joint.bounds.center.to_a
					break
				end
			end
		end
		return center
	end
	
	class Skel::Bone
		attr_accessor :name
	end

	def create_bones(parent_node, parent_bone, keyframes)

		# Set keyframe for parent_bone
		if keyframes.has_key?(parent_node.name)
			for frame in keyframes[parent_node.name]
				parent_bone.add_keyframe frame[0], frame[1], frame[2]
				
			end
		end
		parent_bone.name = parent_node.name
		
		# Create child bones
		parent_node.children.each do |child|
			child_bone = parent_bone.add_bone child.content.definition, child.content.transformation, find_joint(child.content)
			create_bones(child, child_bone, keyframes)
			
			child_bone.name = child.name
			
		end
	end

	def create_skeleton(root_node, keyframes, name)

		# Create skeleton, root bone, and child bones
		s = Skel::Skeleton.new name
		root_bone = s.set_root root_node.content.definition, root_node.content.transformation
		create_bones(root_node, root_bone, keyframes)
		return s
	end
end